@@ -259,6 +259,8 @@ function PMA_langDetails($lang)
         return array('en|english', 'en', '');
     case 'en_GB':
         return array('en[_-]gb|english (United Kingdom)', 'en-gb', '');
+    case 'eo':
+        return array('eo|esperanto', 'eo', 'Esperanto');
     case 'es':
         return array('es|spanish', 'es', 'Espa&ntilde;ol');
     case 'et':
@@ -271,6 +273,8 @@ function PMA_langDetails($lang)
         return array('fi|finnish', 'fi', 'Suomi');
     case 'fr':
         return array('fr|french', 'fr', 'Fran&ccedil;ais');
+    case 'fy':
+        return array('fy|frisian', 'fy', 'Frysk');
     case 'gl':
         return array('gl|galician', 'gl', 'Galego');
     case 'he':
@@ -309,6 +313,8 @@ function PMA_langDetails($lang)
         return array('ksh|colognian', 'ksh', 'Kölsch');
     case 'ky':
         return array('ky|kyrgyz', 'ky', 'Кыргызча');
+    case 'li':
+        return array('li|limburgish', 'li', 'Lèmbörgs');
     case 'lt':
         return array('lt|lithuanian', 'lt', 'Lietuvi&#371;');
     case 'lv':
@@ -600,36 +606,16 @@ $GLOBALS['l']['w_page'] = __('Page number:');
 
 
 // now, that we have loaded the language strings we can send the errors
-if ($GLOBALS['lang_failed_cfg']) {
+if ($GLOBALS['lang_failed_cfg']
+    || $GLOBALS['lang_failed_cookie']
+    || $GLOBALS['lang_failed_request']
+) {
     trigger_error(
-        sprintf(
-            __('Unknown language: %1$s.'),
-            htmlspecialchars($GLOBALS['lang_failed_cfg'])
-        ),
+        __('Ignoring unsupported language code.'),
         E_USER_ERROR
     );
 }
-if ($GLOBALS['lang_failed_cookie']) {
-    trigger_error(
-        sprintf(
-            __('Unknown language: %1$s.'),
-            htmlspecialchars($GLOBALS['lang_failed_cookie'])
-        ),
-        E_USER_ERROR
-    );
-}
-if ($GLOBALS['lang_failed_request']) {
-    trigger_error(
-        sprintf(
-            __('Unknown language: %1$s.'),
-            htmlspecialchars($GLOBALS['lang_failed_request'])
-        ),
-        E_USER_ERROR
-    );
-}
-
 unset(
     $line, $fall_back_lang, $GLOBALS['lang_failed_cfg'],
     $GLOBALS['lang_failed_cookie'], $GLOBALS['lang_failed_request']
 );
-?>
